#version-warning-banner {
  margin: 10px 0;
  padding: 10px;
  border-radius: 4px;

  letter-spacing: 1px;
  color: #fff;
  text-shadow: 0 0 2px #000;
  text-align: center;
  font-size: 0.9em;

  background: #000
    repeating-linear-gradient(
      135deg,
      transparent,
      transparent 20px,
      rgba(255, 255, 255, 0.1) 20px,
      rgba(255, 255, 255, 0.1) 40px
    );
}

#version-warning-banner .admonition-title {
  font-size: inherit;
  font-weight: bold;
}

#version-warning-banner p.admonition-title::after {
  content: none;
}

#version-warning-banner a {
  color: #fff;
  border-color: #fff;
}

#version-warning-banner p {
  display: inline;
}

div.warning {
  border: 0;
}
